SCAN_TYPE_MAP = {
    205: "Target not Responsive",
    208: "Target Verification Failed",
    209: "Target Continuous Mode Disabled",
    601: "Report Created",
    604: "Report Failed",
    605: "Export Failed",
    400: "Scan Scheduled",
    401: "Scan Modified",
    402: "Scan Deleted",
    403: "Scan Started",
    404: "Scan Done",
    405: "Scan Failed",
    406: "Scan User Aborted",
    407: "Scan Recurrence Suspended",
    410: "Scan Job Starting",
    411: "Scan Job Completed",
    412: "Scan Job Failed",
    413: "Scan Job Aborted",
    415: "Scan Paused",
    420: "Scan Scanner Event",
    430: "Scan Crawler Memory Limited Reached",
    431: "Scan Aborted By Scanner",
    432: "Scan AcuSensor Found",
    433: "Scan AcuSensor Not Found",
    434: "Scan Automatic Login Failed",
    435: "Scan Invalid HTTP LoginDto",
    436: "Scan Login Sequence Required",
    437: "Scan Crawling",
    438: "Scan Deepscan",
    439: "Scan Started",
    440: "Scan Finished",
    441: "Scan Manual Intervention",
    442: "Scan Resumed",
    443: "Initial Request",
    444: "Importer Failed",
    445: "Target not responsive",
    446: "LSR/Autologin not configured",
    502: "Vulnerability discovered by AcuMonitor",
    801: "Engine is not responding",
    802: "Engine is offline",
    803: "Engine is back online",
}

SCAN_DONE_STATUS = [404, 405, 406, 407, 412, 413, 431, 440, 445]
REPORT_DONE_STATUS = [601, 604]
EXPORT_DONE_STATUS = [605]
